/*
 * Copyright (c) 2012 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*jslint devel:true*/
/*global tizen */

/**
 * @class Model
 */
var Model = function Model() {
    'use strict';
    return;
};

(function strict() { // strict mode wrapper
    'use strict';
    Model.prototype = {

        /**
         * @type Long
         */
        listenerId: 0,
        /**
         * Model module initialisation
         * @param {object} app Parent object.
         */
        init: function Model_init(app) {
            this.app = app;
            this.listenerId = this.addEventListener();
        },

        /**
         * Creates a TimeDuration object corresponding to the given duration
         * in minutes.
         *
         * @param {int} minutes Duration in minutes
         * @return {TimeDuration}
         */
        getTimeDuration: function Model_getTimeDuration(minutes) {
            if (minutes === 0) {
                return new tizen.TimeDuration(minutes, 'MINS');
            }
            if (minutes % 1440 === 0) {
                return new tizen.TimeDuration(minutes / 1440, 'DAYS');
            }
            if (minutes % 60 === 0) {
                return new tizen.TimeDuration(minutes / 60, 'HOURS');
            }
            return new tizen.TimeDuration(minutes, 'MINS');
        },

        /**
         * Creates a CalendarAlarm with the given duration.
         *
         * @param {int} minutes Alarm duration in minutes.
         * @param {string} description Alarm description.
         * @return {CalendarAlarm}
         */
        getCalendarAlarm:
            function Model_getCalendarAlarm(minutes, description) {
                var timeDuration,
                    calendarAlarm;

                timeDuration = this.getTimeDuration(minutes);
                calendarAlarm = new tizen.CalendarAlarm(timeDuration, 'DISPLAY',
                    description);
                return calendarAlarm;
            },

        /**
         * Creates a new event and adds it to the default calendar
         *
         * @param {CalendarEventInit} init
         * @config {TZDate} init.startDate
         * @config {bool} init.isAllDay
         * @config {TimeDuration} init.duration
         * @config {string} init.summary
         * @config {string} init.description
         * @config {string} init.location
         */
        addEventToDefaultCalendar:
            function Model_addEventToDefaultCalendar(init) {
                var calendar = null,
                    event = null;
                calendar = this.getCalendar();
                event = new tizen.CalendarEvent(init);
                calendar.add(event);
            },

        /**
         * Returns the default event calendar.
         * @return {Calendar}
         */
        getCalendar: function Model_getCalendar() {
            // get the default calendar
            return tizen.calendar.getDefaultCalendar('EVENT');
        },

        /**
         * Finds all events in the default calendar on the given date.
         *
         * @param {Date} date Date (in local time).
         * @param {function} onSuccess Success callback.
         * @param {function} onError Error callback.
         */
        getEventsFromDefaultCalendar:
            function Model_getEventsFromDefaultCalendar(date, onSuccess,
                onError) {
                var calendar = null, filter = null;

                calendar = this.getCalendar();

                // show all events
                filter = this.getDateFilter(date);

                calendar.find(onSuccess, onError, filter);
            },


        /**
         * Creates an intersection of filters.
         * @param {AbstractFilter[]} arr
         * @returns {CompositeFilter}
         */
        filtersIntersection: function Model_filtersIntersection(arr) {
            return new tizen.CompositeFilter(
                'INTERSECTION',
                arr
            );
        },

        /**
         * Creates a union of filters.
         * @param {AbstractFilter[]} arr
         * @returns {CompositeFilter}
         */
        filtersUnion: function Model_filtersIntersection(arr) {
            return new tizen.CompositeFilter(
                'UNION',
                arr
            );
        },

        /**
         * Creates a filter for selecting events on the given day.
         *
         * @param {Date} date
         * @return {CompositeFilter}
         */
        getDateFilter: function Model_getDateFilter(date) {
            var filters = {},
                beginOfToday = new tizen.TZDate(date.getFullYear(),
                    date.getMonth(),
                    date.getDate(),
                    0, 0, 0, 0
                    ),
                endOfToday = new tizen.TZDate(date.getFullYear(),
                    date.getMonth(),
                    date.getDate(),
                    23, 59, 59, 999
                    );

            filters.notAllDay = new tizen.AttributeFilter(
                'isAllDay',
                'EXACTLY',
                false
            );

            filters.isAllDay = new tizen.AttributeFilter(
                'isAllDay',
                'EXACTLY',
                true
            );

            filters.startsBeforeEndOfToday = new tizen.AttributeRangeFilter(
                'startDate',
                null,
                endOfToday
            );

            filters.endsAfterBeginOfToday = new tizen.AttributeRangeFilter(
                'endDate',
                beginOfToday,
                null
            );

            /**
             * All-day events ocurring at the given day must both
             * - start before the end of the day
             * - end after the end of the next day (ie. after at least 1 day)
             * Otherwise they are either entirely occuring after or before
             * the given date.
             */
            filters.matchingAllDay = this.filtersIntersection([
                filters.isAllDay,
                filters.startsBeforeEndOfToday,
                filters.endsAfterBeginOfToday
            ]);

            /**
             * Other events to occur on the given day must both
             * - start before the end of the day (same condition as before)
             * - end after the start of the day
             */
            filters.matchingNotAllDay = this.filtersIntersection([
                filters.notAllDay,
                filters.startsBeforeEndOfToday,
                filters.endsAfterBeginOfToday
            ]);

            // Unify, because we need them both
            return this.filtersUnion([
                filters.matchingAllDay,
                filters.matchingNotAllDay
            ]);
        },

        /**
         * Checks if the event with given id exists.
         *
         * @param {EventId} eventId
         * @param {function} success
         * @param {function} error
         */
        eventExists: function eventExists(eventId, success, error) {
            var calendar = this.getCalendar();
            error = error || function noop() { return; };
            success = success || function noop() { return; };
            try {
                calendar.get(new tizen.CalendarEventId(eventId));
                success();
            } catch (e) {
                error(e);
            }
        },

        /**
         * Deletes the event with the given id.
         * @param {string} eventId
         */
        deleteEvent: function Model_deleteEvent(eventId) {
            var calendar = this.getCalendar();
            calendar.remove(new tizen.CalendarEventId(eventId));
            this.app.loadEvents(); // reload the list
        },

        /**
         * @param {string} eventId
         */
        editEvent: function Model_editEvent(eventId) {
            var calendar = this.getCalendar();
            return calendar.get(new tizen.CalendarEventId(eventId));
        },

        /**
         * Updates event using the given new properties values.
         * @param {string} eventId
         * @param {object} newValues New values for the event properties.
         */
        updateEvent: function Model_updateEvent(eventId, newValues) {
            var calendar = this.getCalendar(),
                prop,
                event = calendar.get(new tizen.CalendarEventId(eventId));

            for (prop in newValues) {
                if (newValues.hasOwnProperty(prop)) {
                    // copy new values into the event object
                    event[prop] = newValues[prop];
                }
            }

            calendar.update(event); // save to myCalendar
        },

        /**
         * Creates a TZDate object with timezone info from a Date object.
         * @param {Date} date
         * @return {TZDate} date with timezone info
         */
        createTZDateFromDate: function Model_createTZDateFromDate(date) {
            return new tizen.TZDate(date.getFullYear(),
                                    date.getMonth(),
                                    date.getDate(),
                                    date.getHours(),
                                    date.getMinutes(),
                                    date.getSeconds(),
                                    date.getMilliseconds());
        },

        /**
         * Adds event change listener.
         * @return {number} Listener id.
         */
        addEventListener: function Model_addEventListener() {
            var calendar = this.getCalendar(),
                // Params in listeners are ignored
                // because the whole list is reloaded anyway,
                listener = {
                    onitemsadded: function onItemsAdded() {
                        this.app.loadEvents(); // reload the list
                    },
                    onitemsupdated: function onItemsUpdated() {
                        this.app.loadEvents(); // reload the list
                    },
                    onitemsremoved: function onItemsRemoved() {
                        this.app.loadEvents(); // reload the list
                        this.app.checkDeletePopup();
                    }
                };
            return calendar.addChangeListener(listener);
        },
        /**
         * Exits from the application.
         */
        exit: function Model_exit() {
            var calendar = this.getCalendar();

            calendar.removeChangeListener(this.listenerId);

            tizen.application.getCurrentApplication().exit();
        }
    };
}());


